import 'package:flutter/material.dart';
import 'package:flutter/services.dart';
import 'package:flutter/widgets.dart';
import 'package:intl/intl.dart' hide TextDirection;
import '../controls/row_spacing.dart';

class CounterFormField extends FormField<num> {
  CounterFormField({
    Key key,
    this.step = 1,
    this.fractionDigits = 0,
    this.minValue = 0,
    this.maxValue,
    this.controller,
    String labelText,
    String hintText,
    num initialValue,
    FocusNode focusNode,
    TextInputAction textInputAction,
    TextAlign textAlign = TextAlign.start,
    VoidCallback onEditingComplete,
    ValueChanged<num> onFieldSubmitted,
    FormFieldSetter<num> onSaved,
    FormFieldValidator<num> validator,
    bool enabled = true,
    double spacingFlactor = 2,
    EdgeInsetsGeometry contentPadding = const EdgeInsets.all(8),
    EdgeInsetsGeometry margin = const EdgeInsets.symmetric(vertical: 8),
  })  : assert(initialValue == null || controller == null),
        super(
          key: key,
          initialValue: controller != null ? controller.value : initialValue,
          onSaved: onSaved,
          validator: validator,
          enabled: enabled,
          builder: (field) {
            final _CounterFormFieldState state = field;
            final effectiveDecoration = InputDecoration(
              contentPadding: contentPadding,
            ).copyWith(
                hintText: hintText,
                border: OutlineInputBorder(borderSide: BorderSide.none));
            final color = Theme.of(state.context).brightness == Brightness.light
                ? Colors.grey.shade700
                : Colors.white70;
            return Padding(
                padding: margin,
                child: Column(
                    crossAxisAlignment: CrossAxisAlignment.stretch,
                    children: [
                      Text(
                        labelText,
                        textAlign: TextAlign.left,
                        style: Theme.of(state.context).textTheme.subtitle1,
                      ),
                      RowSpacing(spacingFlactor: spacingFlactor),
                      Container(
                          color: Colors.white,
                          child: InputDecorator(
                              decoration: effectiveDecoration.copyWith(
                                  errorText: field.errorText),
                              isEmpty: field.value == null,
                              child: Row(
                                children: <Widget>[
                                  Expanded(
                                    child: TextField(
                                      inputFormatters: [
                                        FilteringTextInputFormatter.allow(
                                            RegExp("[0-9.]"))
                                      ],
                                      controller: TextEditingController(
                                          text: state._effectiveController.value
                                              .toStringAsFixed(fractionDigits)),
                                      focusNode: focusNode,
                                      decoration: null,
                                      keyboardType: TextInputType.number,
                                      textInputAction: textInputAction,
                                      textAlign: textAlign,
                                      textCapitalization:
                                          TextCapitalization.none,
                                      autofocus: false,
                                      autocorrect: true,
                                      onChanged: (v) {
                                        field.didChange((v == null || v.isEmpty)
                                            ? minValue
                                            : num.parse(v));
                                      },
                                      onEditingComplete: onEditingComplete,
                                      onSubmitted: (v) {
                                        onFieldSubmitted(
                                            (v == null || v.isEmpty)
                                                ? minValue
                                                : num.parse(v));
                                      },
                                      enabled: enabled,
                                    ),
                                  ),
                                  InkWell(
                                    onTap: () {
                                      state.changeValue(state.value - step);
                                    },
                                    child: Icon(
                                      Icons.remove,
                                      color: color,
                                    ),
                                  ),
                                  InkWell(
                                    onTap: () {
                                      state.changeValue(state.value + step);
                                    },
                                    child: Icon(
                                      Icons.add,
                                      color: color,
                                    ),
                                  ),
                                ],
                              )))
                    ]));
          },
        );

  /// Controls the text being edited.
  ///
  /// If null, this widget will create its own [TextEditingController] and
  /// initialize its [TextEditingController.text] with [initialValue].
  final num step;
  final int fractionDigits;
  final num minValue;
  final num maxValue;
  final NumEditingController controller;

  @override
  _CounterFormFieldState createState() => _CounterFormFieldState();
}

class _CounterFormFieldState extends FormFieldState<num> {
  NumEditingController _controller;

  NumEditingController get _effectiveController =>
      widget.controller ?? _controller;

  @override
  CounterFormField get widget => super.widget;

  @override
  void initState() {
    super.initState();
    if (widget.controller == null) {
      _controller = NumEditingController(value: widget.initialValue);
    } else {
      widget.controller.addListener(_handleControllerChanged);
    }
  }

  void changeValue(num v) {
    final va = v < widget.minValue
        ? widget.minValue
        : ((widget.maxValue != null && v > widget.maxValue)
            ? widget.maxValue
            : v);
    _effectiveController.value = va;
    didChange(va);
  }

  @override
  void didUpdateWidget(CounterFormField oldWidget) {
    super.didUpdateWidget(oldWidget);
    if (widget.controller != oldWidget.controller) {
      oldWidget.controller?.removeListener(_handleControllerChanged);
      widget.controller?.addListener(_handleControllerChanged);

      if (oldWidget.controller != null && widget.controller == null) {
        _controller =
            NumEditingController.fromValue(oldWidget.controller.value);
      }
      if (widget.controller != null) {
        setValue(widget.controller.value);
        if (oldWidget.controller == null) _controller = null;
      }
    }
  }

  @override
  void dispose() {
    widget.controller?.removeListener(_handleControllerChanged);
    super.dispose();
  }

  @override
  void reset() {
    super.reset();
    setState(() {
      _effectiveController.value = widget.initialValue;
    });
  }

  void _handleControllerChanged() {
    if (_effectiveController.value != value) {
      didChange(_effectiveController.value);
    }
  }
}

class NumericTextFormatter extends TextInputFormatter {
  NumericTextFormatter({this.fractionDigits = 0});

  int fractionDigits;

  TextEditingValue formatEditUpdate(
      TextEditingValue oldValue, TextEditingValue newValue) {
    if (newValue.text.isEmpty) {
      return newValue.copyWith(text: '');
    } else if (newValue.text.compareTo(oldValue.text) != 0) {
      var selectionIndexFromTheRight =
          newValue.text.length - newValue.selection.end;
      var s = '';
      for (var i = 0; i < fractionDigits; i++) {
        s += '#';
      }
      final f = NumberFormat("#,###${s.isNotEmpty ? '.$s' : ''}");
      var num = int.parse(newValue.text.replaceAll(f.symbols.GROUP_SEP, ''));
      final newString = f.format(num);
      return TextEditingValue(
        text: newString,
        selection: TextSelection.collapsed(
            offset: newString.length - selectionIndexFromTheRight),
      );
    } else {
      return newValue;
    }
  }
}

class NumEditingController extends ValueNotifier<num> {
  NumEditingController({num value}) : super(value ?? 0);

  NumEditingController.fromValue(num value) : super(value ?? 0);
}
