import 'package:flutter/cupertino.dart';
import 'package:flutter/material.dart';
import 'package:mobile_ui/mobile_ui.dart';

class DatePickerFormField extends FormField<DateTime> {
  final DateTime firstDate;
  final DateTime lastDate;
  final bool canClear;
  final CupertinoDatePickerMode mode;

  static Future<void> _selectDate(
      BuildContext context,
      DateTime initialValue,
      DateTime firstDate,
      DateTime lastDate,
      ValueChanged<DateTime> onChanged,
      CupertinoDatePickerMode mode) async {
    final dateTime = await showModalBottomSheet(
        context: context,
        builder: (_) {
          DateTime dt;
          return Padding(
            padding: EdgeInsets.symmetric(vertical: 8),
            child: Column(
              children: [
                Row(
                  mainAxisAlignment: MainAxisAlignment.spaceBetween,
                  children: [
                    TextButton(
                        onPressed: () => Navigator.of(context).pop(),
                        child: Text('Cancel')),
                    TextButton(
                        onPressed: () => Navigator.of(context).pop(dt),
                        child: Text('Save')),
                  ],
                ),
                Expanded(
                    child: CupertinoDatePicker(
                  mode: mode ?? CupertinoDatePickerMode.date,
                  maximumDate: lastDate,
                  minimumDate: firstDate,
                  initialDateTime: initialValue,
                  onDateTimeChanged: (value) {
                    dt = value;
                  },
                ))
              ],
            ),
          );
        });
    if (dateTime != null && dateTime != initialValue) onChanged(dateTime);
  }

  DatePickerFormField(
      {DateTime initialValue,
      this.controller,
      this.canClear = false,
      this.firstDate,
      this.lastDate,
      this.mode = CupertinoDatePickerMode.dateAndTime,
      String labelText,
      String hintText,
      String format = formatDateTimeSG,
      FormFieldSetter<DateTime> onSaved,
      FormFieldValidator<DateTime> validator,
      EdgeInsetsGeometry contentPadding = const EdgeInsets.all(8),
      EdgeInsetsGeometry margin = const EdgeInsets.symmetric(vertical: 8),
      double spacingFlactor = 2,
      bool isEmpty})
      : assert(initialValue == null || controller == null),
        super(
            onSaved: onSaved,
            validator: validator,
            initialValue: controller != null ? controller.value : initialValue,
            builder: (field) {
              final _DatePickerFormFieldState state = field;
              final effectiveDecoration = InputDecoration(
                contentPadding: contentPadding,
              ).copyWith(
                  hintText: hintText,
                  border: OutlineInputBorder(borderSide: BorderSide.none));
              final color =
                  Theme.of(state.context).brightness == Brightness.light
                      ? Colors.grey.shade700
                      : Colors.white70;

              var text =
                  (field.value == null) ? '' : field.value.formatBy(format);
              final icon = (canClear && field.value != null)
                  ? InkWell(
                      onTap: () {
                        field.didChange(null);
                      },
                      child: Icon(
                        Icons.clear,
                        color: color,
                      ),
                    )
                  : Icon(
                      Icons.arrow_drop_down,
                      color: color,
                    );
              var children = <Widget>[];
              if (labelText != null && labelText.isNotEmpty) {
                children.addAll([
                  Text(
                    labelText,
                    textAlign: TextAlign.left,
                    style: Theme.of(state.context).textTheme.subtitle1,
                  ),
                  RowSpacing(
                    spacingFlactor: 2,
                  )
                ]);
              }
              return Padding(
                  padding: margin,
                  child: Column(
                      crossAxisAlignment: CrossAxisAlignment.stretch,
                      children: [
                        ...children,
                        InkWell(
                            onTap: () {
                              _selectDate(field.context, field.value, firstDate,
                                  lastDate, (dt) {
                                field.didChange(dt);
                              }, mode);
                            },
                            child: Container(
                              color: Colors.white,
                              child: InputDecorator(
                                decoration: effectiveDecoration.copyWith(
                                    errorText: field.errorText),
                                child: Row(
                                  mainAxisAlignment:
                                      MainAxisAlignment.spaceBetween,
                                  mainAxisSize: MainAxisSize.min,
                                  children: <Widget>[
                                    Text(
                                      text,
                                      style: Theme.of(state.context)
                                          .textTheme
                                          .subtitle1,
                                    ),
                                    icon
                                  ],
                                ),
                              ),
                            ))
                      ]));
            });

  final DatePickerEditingController controller;

  @override
  _DatePickerFormFieldState createState() => _DatePickerFormFieldState();
}

class _DatePickerFormFieldState extends FormFieldState<DateTime> {
  DatePickerEditingController _controller;

  DatePickerEditingController get _effectiveController =>
      widget.controller ?? _controller;

  @override
  DatePickerFormField get widget => super.widget;

  @override
  void initState() {
    super.initState();
    if (widget.controller == null) {
      _controller = DatePickerEditingController(value: widget.initialValue);
    } else {
      widget.controller.addListener(_handleControllerChanged);
    }
  }

  @override
  void didUpdateWidget(DatePickerFormField oldWidget) {
    super.didUpdateWidget(oldWidget);
    if (widget.controller != oldWidget.controller) {
      oldWidget.controller?.removeListener(_handleControllerChanged);
      widget.controller?.addListener(_handleControllerChanged);

      if (oldWidget.controller != null && widget.controller == null) {
        _controller =
            DatePickerEditingController.fromValue(oldWidget.controller.value);
      }
      if (widget.controller != null) {
        setValue(widget.controller.value);
        if (oldWidget.controller == null) _controller = null;
      }
    }
  }

  @override
  void dispose() {
    widget.controller?.removeListener(_handleControllerChanged);
    super.dispose();
  }

  @override
  void reset() {
    super.reset();
    setState(() {
      _effectiveController.value = widget.initialValue;
    });
  }

  void _handleControllerChanged() {
    if (_effectiveController.value != value) {
      didChange(_effectiveController.value);
    }
  }
}

class DatePickerEditingController extends ValueNotifier<DateTime> {
  DatePickerEditingController({DateTime value}) : super(value ?? null);

  DatePickerEditingController.fromValue(DateTime value) : super(value ?? 0);
}
