import 'package:flutter/material.dart';
import '../utils/message.dart';
import 'package:multi_image_picker/multi_image_picker.dart';

class MultiImagePickerFormField extends FormField<List<Asset>> {
  final int maxImage;

  MultiImagePickerFormField(
      {Key key,
      FormFieldSetter<List<Asset>> onSaved,
      FormFieldValidator<List<Asset>> validator,
      this.maxImage = 6,
      this.controller,
      List<DropdownMenuItem<List<Asset>>> items,
      List<Asset> initialValue})
      : super(
            key: key,
            initialValue: controller != null ? controller.value : [],
            onSaved: onSaved,
            validator: validator,
            builder: (field) {
              final _MultiImagePickerFormFieldState state = field;
              return state.buildGridView();
            });

  final MultiImagePickerEditingController controller;

  @override
  FormFieldState<List<Asset>> createState() =>
      _MultiImagePickerFormFieldState();
}

class _MultiImagePickerFormFieldState extends FormFieldState<List<Asset>> {
  @override
  MultiImagePickerFormField get widget => super.widget;
  MultiImagePickerEditingController _controller;

  MultiImagePickerEditingController get _effectiveController =>
      widget.controller ?? _controller;

  @override
  void initState() {
    super.initState();
    if (widget.controller == null) {
      _controller =
          MultiImagePickerEditingController(value: widget.initialValue);
    } else {
      widget.controller.addListener(_handleControllerChanged);
    }
  }

  @override
  void didUpdateWidget(MultiImagePickerFormField oldWidget) {
    super.didUpdateWidget(oldWidget);
    if (widget.controller != oldWidget.controller) {
      oldWidget.controller?.removeListener(_handleControllerChanged);
      widget.controller?.addListener(_handleControllerChanged);

      if (oldWidget.controller != null && widget.controller == null) {
        _controller = MultiImagePickerEditingController.fromValue(
            oldWidget.controller.value);
      }
      if (widget.controller != null) {
        setValue(widget.controller.value);
        if (oldWidget.controller == null) _controller = null;
      }
    }
  }

  @override
  void dispose() {
    widget.controller?.removeListener(_handleControllerChanged);
    super.dispose();
  }

  @override
  void reset() {
    super.reset();
    setState(() {
      _effectiveController.value = widget.initialValue;
    });
  }

  void _handleControllerChanged() {
    if (_effectiveController.value != value) {
      didChange(_effectiveController.value);
    }
  }

  Widget buildGridView() {
    var children = buildImages();
    final color = Theme.of(context).brightness == Brightness.light
        ? Colors.grey.shade700
        : Colors.white70;
    children.add(InkWell(
        onTap: loadAssets,
        child: Container(
            alignment: Alignment.center,
            width: 90,
            height: 90,
            decoration: BoxDecoration(
              border: Border.all(color: color),
            ),
            child: Column(
              mainAxisAlignment: MainAxisAlignment.spaceEvenly,
              children: <Widget>[
                Icon(
                  Icons.camera_enhance,
                  size: 40,
                ),
                Text(
                  "Upload images",
                  style: TextStyle(fontSize: 12),
                )
              ],
            ))));
    return Padding(
        padding: EdgeInsets.only(top: 10, bottom: 20),
        child: Wrap(
          children: children,
          spacing: 5,
          runSpacing: 5,
        ));
  }

  List<Widget> buildImages() {
    if (value != null) {
      return List.generate(value.length, (index) {
        var asset = value[index];
        return Stack(alignment: AlignmentDirectional.bottomEnd, children: [
          AssetThumb(
            asset: asset,
            width: 90,
            height: 90,
          ),
          InkWell(
              onTap: () {
                var resultList = value;
                resultList.remove(asset);
                didChange(resultList);
              },
              child: Icon(
                Icons.delete_outline,
                color: Colors.red,
              ))
        ]);
      });
    }
    return <Widget>[];
  }

  Future<void> loadAssets() async {
    List<Asset> resultList;
    var color =
        '#${Theme.of(context).primaryColor.value.toRadixString(16).padLeft(8, '0').substring(2)}';
    try {
      resultList = await MultiImagePicker.pickImages(
        maxImages: widget.maxImage,
        enableCamera: true,
        cupertinoOptions: CupertinoOptions(takePhotoIcon: "chat"),
        materialOptions: MaterialOptions(
          actionBarColor: color,
          actionBarTitle: "Upload Images",
          allViewTitle: "All Photos",
          startInAllView: true,
          textOnNothingSelected: null,
          useDetailsView: false,
        ),
      );
    } on Exception catch (e) {
      showError(e.toString());
    }

    // If the widget was removed from the tree while the asynchronous platform
    // message was in flight, we want to discard the reply rather than calling
    // setState to update our non-existent appearance.
    if (!mounted) return;
    if (resultList != null) {
      resultList.insertAll(0, value);
      didChange(resultList);
    }
  }
}

class MultiImagePickerEditingController extends ValueNotifier<List<Asset>> {
  MultiImagePickerEditingController({List<Asset> value}) : super(value ?? []);

  MultiImagePickerEditingController.fromValue(List<Asset> value)
      : super(value ?? 0);
}
