import 'package:flutter/material.dart';
import 'package:validators/validators.dart';

import '../../mobile_ui.dart';

enum TextType { text, password, email, url, phone, ip, money }

class TextFormInput extends StatelessWidget {
  final TextEditingController controller;
  final String labelText;
  final String hintText;
  final bool isRequired;
  final TextType textType;
  final FormFieldSetter<String> onSaved;
  final bool readOnly;
  final bool enabled;
  final EdgeInsetsGeometry padding;
  final EdgeInsetsGeometry margin;
  final int minLength;
  final int maxLength;
  final FocusNode focusNode;
  final TextInputAction textInputAction;
  final String initialValue;
  final Icon icon;
  final bool autofocus;
  final VoidCallback onEditingComplete;
  final ValueChanged<String> onSubmitted;
  final int maxLines;
  final String Function(String value, String error, String errorMsg) validator;

  TextFormInput(
      {Key key,
      this.initialValue,
      this.controller,
      this.labelText,
      this.hintText,
      this.isRequired = false,
      this.textType = TextType.text,
      this.readOnly = false,
      this.enabled = true,
      this.onSaved,
      this.minLength,
      this.maxLength,
      this.focusNode,
      this.textInputAction,
      this.icon,
      this.autofocus = false,
      this.onEditingComplete,
      this.onSubmitted,
      this.padding = const EdgeInsets.all(8),
      this.margin = const EdgeInsets.symmetric(vertical: 8),
      this.maxLines,
      this.validator})
      : super(key: key);

  @override
  Widget build(BuildContext context) {
    var obscureText = false;
    TextInputType keyboardType;
    switch (textType) {
      case TextType.password:
        {
          keyboardType = TextInputType.visiblePassword;
          obscureText = true;
        }
        break;
      case TextType.email:
        {
          keyboardType = TextInputType.emailAddress;
        }
        break;
      case TextType.url:
        {
          keyboardType = TextInputType.url;
        }
        break;
      case TextType.phone:
        {
          keyboardType = TextInputType.phone;
        }
        break;
      case TextType.ip:
        {
          keyboardType = TextInputType.number;
        }
        break;
      case TextType.money:
        {
          keyboardType =
              TextInputType.numberWithOptions(decimal: true, signed: true);
        }
        break;
      default:
        {
          keyboardType = TextInputType.text;
        }
        break;
    }
    final textField = Container(
        color: Colors.white,
        child: StatefulBuilder(builder: (context, setState) {
          final theme = Theme.of(context);
          var inputDecoration = InputDecoration(
            contentPadding: padding,
            icon: icon,
            suffix: textType == TextType.password
                ? InkWell(
                    onTap: () {
                      if (!enabled) return;
                      setState(() =>
                          obscureText = obscureText == true ? false : true);
                    },
                    child: Icon(
                        obscureText == true
                            ? Icons.visibility
                            : Icons.visibility_off,
                        color:
                            enabled ? theme.primaryColor : theme.disabledColor),
                  )
                : null,
          );
          InputCounterWidgetBuilder builderCounter;
          var _maxLines = maxLines ?? 1;
          inputDecoration = inputDecoration.copyWith(
            hintText: hintText,
            hintMaxLines: 2,
            border: OutlineInputBorder(borderSide: BorderSide.none),
          );
          if (_maxLines > 1) {
            keyboardType = TextInputType.multiline;
            if (maxLength != null) {
              builderCounter = (context,
                  {@required currentLength,
                  @required maxLength,
                  @required isFocused}) {
                var style = enabled
                    ? theme.textTheme.caption
                    : theme.textTheme.caption
                        .copyWith(color: theme.disabledColor);
                return Text(
                  "$currentLength / $maxLength",
                  style: style,
                );
              };
            }
          }
          return TextFormField(
              initialValue: initialValue,
              autofocus: autofocus,
              controller: controller,
              keyboardType: keyboardType,
              maxLines: _maxLines,
              obscureText: obscureText,
              readOnly: readOnly,
              enabled: enabled,
              decoration: inputDecoration,
              focusNode: focusNode,
              textInputAction: textInputAction ??
                  (_maxLines > 1
                      ? TextInputAction.newline
                      : TextInputAction.next),
              maxLength: maxLength,
              buildCounter: builderCounter,
              validator: (value) {
                String errorCode;
                String errorMsg;
                if (value == null || value.isEmpty) {
                  if (isRequired == true) {
                    errorCode = 'required';
                    errorMsg = '${labelText ?? "This field"} is required.';
                  }
                } else {
                  if (minLength != null && value.length < minLength) {
                    errorCode = 'lengthNotEnough';
                    errorMsg = '${labelText ?? "This field"} is required.';
                  } else if (maxLength != null && value.length > maxLength) {
                    errorCode = 'lengthExceeded';
                    errorMsg =
                        '${labelText ?? "This field"} cannot have less than $minLength characters.';
                  } else if (textType == TextType.email && !isEmail(value)) {
                    errorCode = 'invalidEmail';
                    errorMsg =
                        '${labelText ?? "This field"} requires a valid email address.';
                  } else if (textType == TextType.url && !isURL(value)) {
                    errorCode = 'invalidUrl';
                    errorMsg =
                        '${labelText ?? "This field"} requires a valid url address.';
                  } else if (textType == TextType.ip && !isIP(value)) {
                    errorCode = 'invalidIp';
                    errorMsg =
                        '${labelText ?? "This field"} requires a valid ip address.';
                  } else if (textType == TextType.phone && !isNumeric(value)) {
                    errorCode = 'invalidPhone';
                    errorMsg =
                        '${labelText ?? "This field"} requires a valid phone number.';
                  }
                }
                if (validator != null)
                  return validator(value, errorCode, errorMsg);
                return errorMsg;
              },
              onFieldSubmitted:
                  onSubmitted ?? (_) => FocusScope.of(context).nextFocus(),
              onSaved: onSaved,
              onEditingComplete: onEditingComplete);
        }));
    final children = <Widget>[];
    if (labelText != null && labelText.isNotEmpty) {
      children.addAll([
        Text(
          labelText,
          textAlign: TextAlign.left,
          style: Theme.of(context).textTheme.subtitle1,
        ),
        RowSpacing()
      ]);
    }
    children.add(textField);
    return Padding(
      padding: margin,
      child: Column(
        crossAxisAlignment: CrossAxisAlignment.stretch,
        children: children,
      ),
    );
  }
}
